using System;
using System.IO;
using System.Text;
using System.Web;
using System.Xml;

/// <summary>
/// Increments a counter and forces a download to the user. 
/// </summary>
public class DownloadHandler : IHttpHandler
{
    private readonly string COUNTER_REPOSITORY = "App_Data/ExampleCounter.xml";
    private readonly string MY_FILE = "App_Data/Example.pdf";
    private readonly string SAMPLE_DOWNLOAD_COUNT_ELEMENT = "SampleDownloadCount";

    #region IHttpHandler Members

    ///<summary>
    ///Enables processing of HTTP Web requests 
    /// by a custom HttpHandler that implements 
    /// the <see cref="T:System.Web.IHttpHandler"></see> interface.
    ///</summary>
    ///<param name="context">An 
    /// <see cref="T:System.Web.HttpContext"></see> object that provides 
    /// references to the intrinsic server objects (for example, Request, 
    /// Response, Session, and Server) used to service HTTP requests. </param>
    public void ProcessRequest(HttpContext context)
    {
        IncrementCounter(context);
        SendFileToUser(context);
    }

    ///<summary>
    ///Gets a value indicating whether another request can use 
    /// the <see cref="T:System.Web.IHttpHandler"></see> instance.
    ///</summary>
    ///
    ///<returns>
    ///true if the <see cref="T:System.Web.IHttpHandler"></see> instance 
    /// is reusable; otherwise, false.
    ///</returns>
    ///
    public bool IsReusable
    {
        get { return false; }
    }

    #endregion

    /// <summary>
    /// Sends the file to the user. 
    /// </summary>
    /// <param name="context">The <see cref="HttpContext"/> that 
    /// contains the request.</param>
    private void SendFileToUser(HttpContext context)
    {
        // Send the file to the user.
        FileInfo fileInfo = new FileInfo(context.Server.MapPath(MY_FILE));

        context.Response.ContentType = "application/pdf";
        context.Response.AddHeader(
            "Content-Disposition", 
            "attachment; filename=" + fileInfo.Name);
        context.Response.WriteFile(fileInfo.FullName);
        context.Response.End();
    }

    /// <summary>
    /// Increments the counter. 
    /// </summary>
    /// <param name="context">The <see cref="HttpContext"/> that contains t
    /// he current request</param>
    private void IncrementCounter(HttpContext context)
    {
        int value = int.MinValue;
        string fileName = context.Server.MapPath(COUNTER_REPOSITORY);
        using (XmlTextReader reader = new XmlTextReader(fileName))
        {
            reader.ReadStartElement(SAMPLE_DOWNLOAD_COUNT_ELEMENT);
            value = Convert.ToInt32(reader.ReadString());
        }
        value++;
        using (XmlTextWriter writer = new XmlTextWriter(fileName, Encoding.ASCII))
        {
            writer.WriteElementString(
                SAMPLE_DOWNLOAD_COUNT_ELEMENT, 
                value.ToString());
        }
    }

    


}
